"""Class implementation for Goodreads reviews"""

class GoodreadsReview():
    def __init__(self, review_dict):
        self._review_dict = review_dict

    def __repr__(self):
        return "review [%s]" % self.gid

    @property
    def gid(self):
        """Goodreads id for the review"""
        return self._review_dict['id']

    @property
    def book(self):
        """Book that the review belongs to"""
        return self._review_dict['book']

    @property
    def rating(self):
        """Rating of the book"""
        return self._review_dict['rating']

    @property
    def shelves(self):
        """Shelves for the book"""
        return [shelf['@name']
                for shelf in self._review_dict['shelves']['shelf']]

    @property
    def recommended_for(self):
        """Book recommended for"""
        return self._review_dict['recommended_for']

    @property
    def recommended_by(self):
        """Book recommended by"""
        return self._review_dict['recommended_by']

    @property
    def started_at(self):
        """Book started at"""
        return self._review_dict['started_at']

    @property
    def read_at(self):
        """Book read at"""
        return self._review_dict['read_at']

    @property
    def body(self):
        """Body for the review"""
        return self._review_dict['body']

    @property
    def comments_count(self):
        """Number of comments to this review"""
        return self._review_dict['comments_count']

    @property
    def url(self):
        """URL for this comment"""
        return self._review_dict['url']

    @property
    def owned(self):
        """Is the book owned by this user?"""
        return self._review_dict['owned']




import book
import user

class GoodreadsAuthor:
    def __init__(self, author_dict, client):
        self._author_dict = author_dict
        self._client = client

    def __repr__(self):
        return self.name

    @property
    def gid(self):
        """Goodreads id of the author"""
        return self._author_dict['id']

    @property
    def name(self):
        """Author name"""
        return self._author_dict['name']

    @property
    def about(self):
        """About the author"""
        return self._author_dict['about']

    @property
    def books(self):
        """Books of the author"""
        # Goodreads API returns a list if there are more than one books, otherwise,
        # just the OrderedDict.
        if type(self._author_dict['books']['book']) == list:
            return [book.GoodreadsBook(book_dict, self._client)
                    for book_dict in self._author_dict['books']['book']]
        else:
            return [book.GoodreadsBook(self._author_dict['books']['book'],
                                      self._client)]

    @property
    def born_at(self):
        """Author date of birth"""
        return self._author_dict['born_at']

    @property
    def died_at(self):
        """Author date of death"""
        return self._author_dict['died_at']

    @property
    def fans_count(self):
        """Number of fans"""
        return self._author_dict['fans_count']

    @property
    def gender(self):
        """Author gender"""
        return self._author_dict['gender']

    @property
    def hometown(self):
        """Author's hometown"""
        return self._author_dict['hometown']

    @property
    def link(self):
        """Link for author page"""
        return self._author_dict['link']

    @property
    def image_url(self):
        """Image URL"""
        return self._author_dict['image_url']

    @property
    def small_image_url(self):
        """Small image URL"""
        return self._author_dict['small_image_url']

    @property
    def influences(self):
        """Influenced by"""
        return self._author_dict['influences']

    @property
    def user(self):
        """Goodreads user profile of the author"""
        goodreads_user = None
        if 'user' in self._author_dict:
            goodreads_user = user.GoodreadsUser(
                self._author_dict['user']['id']['#text'], self._client)
        return goodreads_user

    @property
    def works_count(self):
        """Author number of works"""
        return self._author_dict['works_count']



import requests
import xmltodict
import json

class GoodreadsRequestException(Exception):
    def __init__(self, error_msg, url):
        self.error_msg = error_msg
        self.url = url

    def __str__(self):
        return self.url, ':', self.error_msg

class GoodreadsRequest():
    def __init__(self, client, path, query_dict, req_format='xml'):
        """Initialize request object."""
        self.params = dict(query_dict.items() + client.query_dict.items())
        self.host = client.base_url
        self.path = path
        self.req_format = req_format

    def request(self):
        resp = requests.get(self.host+self.path, params=self.params)
        if resp.status_code != 200:
            raise GoodreadsRequestException(resp.reason, self.path)
        if self.req_format == 'xml':
            data_dict = xmltodict.parse(resp.content)
            return data_dict['GoodreadsResponse']
        elif self.req_format == 'json':
            return json.loads(resp.content)
        else:
            raise Exception("Invalid format")



"""Class for GoodreadsGroup"""

class GoodreadsGroup:
    """Goodreads Group class implementation"""
    def __init__(self, group_dict):
        self._group_dict = group_dict

    @property
    def gid(self):
        """Goodreads id for the group"""
        return self._group_dict['id']

    @property
    def title(self):
        """Title for the group"""
        return self._group_dict['title']

    @property
    def description(self):
        """Description of the group"""
        return self._group_dict['description']

    @property
    def category(self):
        """Category of the group"""
        return self._group_dict['category']

    @property
    def subcategory(self):
        """Sub-category of the group"""
        return self._group_dict['subcategory']

    @property
    def rules(self):
        """Group rules"""
        return self._group_dict['rules']

    @property
    def image_url(self):
        """Image URL for the group"""
        return self._group_dict['image_url']

    @property
    def last_activity_at(self):
        """Time of the last activity in the group"""
        return self._group_dict['last_activity_at']

    @property
    def access(self):
        """Is the group public or private?"""
        return self._group_dict['access']

    @property
    def users_count(self):
        """Number of users in the group"""
        return self._group_dict['group_users_count']

    @property
    def members(self):
        """Members of the group"""
        return self._group_dict['members']['group_users']




import webbrowser
from session import GoodreadsSession
from user import GoodreadsUser
from book import GoodreadsBook
from author import GoodreadsAuthor
from request import GoodreadsRequest
from comment import GoodreadsComment
from event import GoodreadsEvent
from group import GoodreadsGroup
from owned_book import GoodreadsOwnedBook
from review import GoodreadsReview
import collections

class GoodreadsClientException(Exception):
    def __init__(self, error_msg):
        self.error_msg = error_msg

    def __str__(self):
        return self.error_msg

class GoodreadsClient():
    base_url = "http://www.goodreads.com/"

    def __init__(self, client_key, client_secret):
        """Initialize the client"""
        self.client_key = client_key
        self.client_secret = client_secret

    @property
    def query_dict(self):
        return {'key': self.client_key}

    def authenticate(self, access_token=None, access_token_secret=None):
        """Authenticate client to query requiring authorization"""
        self.session = GoodreadsSession(self.client_key, self.client_secret,
                                        access_token, access_token_secret)
        if access_token and access_token_secret:
            self.session.oauth_resume()
        else:
            url = self.session.oauth_init()
            webbrowser.open(url)
            while raw_input("Have you authorized me? (y/n)") != 'y':
                pass
            self.session.oauth_finalize()

    def auth_user(self):
        """Return user who authorized OAuth"""
        if not hasattr(self, 'session'):
            raise GoodreadsClientException("No authenticated session")
        resp = self.session.get("api/auth_user", {})
        user_id = resp['user']['@id']
        return self.user(user_id)

    def request(self, *args, **kwargs):
        """Create a GoodreadsRequest object and make that request"""
        req = GoodreadsRequest(self, *args, **kwargs)
        return req.request()

    def request_oauth(self, *args, **kwargs):
        resp = self.session.get(*args, **kwargs)
        return resp

    def user(self, user_id=None, username=None):
        """Get info about a member by id or username.

        If user_id or username not provided, the function returns the authorized
        user
        """
        if not (user_id or username):
            return self.auth_user()
        resp = self.request("user/show", {'id': user_id, 'username': username})
        return GoodreadsUser(resp['user'], self)

    def author(self, author_id):
        """Get info about an author"""
        resp = self.request("author/show", {'id': author_id})
        return GoodreadsAuthor(resp['author'], self)

    def find_author(self, author_name):
        """Find an author by name"""
        resp = self.request("api/author_url/%s" % author_name, {})
        return self.author(resp['author']['@id']) if 'author' in resp else None

    def book(self, book_id=None, isbn=None):
        """Get info about a book"""
        if book_id:
            resp = self.request("book/show", {'id': book_id})
            return GoodreadsBook(resp['book'], self)
        elif isbn:
            resp = self.request("book/isbn", {'isbn': isbn})
            return GoodreadsBook(resp['book'], self)
        else:
            raise GoodreadsClientException("book id or isbn required")

    def search_books(self, q, page=1, search_field='all'):
        """Get the most popular books for the given query. This will search all
        books in the title/author/ISBN fields and show matches, sorted by
        popularity on Goodreads.
        :param q: query text
        :param page: which page to return (default 1)
        :param search_fields: field to search, one of 'title', 'author' or
        'genre' (default is 'all')
        """
        resp = self.request("search/index.xml",
                            {'q': q, 'page': page, 'search[field]': search_field})
        works = resp['search']['results']['work']
        # If there's only one work returned, put it in a list.
        if type(works) == collections.OrderedDict:
            works = [works]
        return [self.book(work['best_book']['id']['#text']) for work in works]

    def group(self, group_id):
        """Get info about a group"""
        resp = self.request("group/show", {'id': group_id})
        return GoodreadsGroup(resp['group'])

    def owned_book(self, owned_book_id):
        """Get info about an owned book, given its id"""
        resp = self.session.get("owned_books/show/%s.xml" % owned_book_id, {})
        return GoodreadsOwnedBook(resp['owned_book']['owned_book'])

    def find_groups(self, query, page=1):
        """Find a group based on the query"""
        resp = self.request("group/search.xml", {'q': query, 'page': page})
        return resp['groups']['list']['group']

    def book_review_stats(self, isbns):
        """Get review statistics for books given a list of ISBNs"""
        resp = self.request("book/review_counts.json",
                            {'isbns': ','.join(isbns)},
                            req_format='json')
        return resp['books']

    def list_comments(self, comment_type, resource_id, page=1):
        """List comments on a subject.

        comment_type should be one of 'author_blog_post', 'blog',
        'book_news_post', 'chapter', 'comment', 'community_answer',
        'event_response', 'fanship', 'friend', 'giveaway', 'giveaway_request',
        'group_user', 'interview', 'librarian_note', 'link_collection', 'list',
        'owned_book', 'photo', 'poll', 'poll_vote', 'queued_item', 'question',
        'question_user_stat', 'quiz', 'quiz_score', 'rating', 'read_status',
        'recommendation', 'recommendation_request', 'review', 'topic', 'user',
        'user_challenge', 'user_following', 'user_list_challenge',
        'user_list_vote', 'user_quote', 'user_status', 'video'.
        """
        resp = self.request("%s/%s/comments" % (comment_type, resource_id),
                            {'format': 'xml'})
        return [GoodreadsComment(comment_dict)
                for comment_dict in resp['comments']['comment']]

    def list_events(self, postal_code):
        """Show events near a location specified with the postal code"""
        resp = self.request("event/index.xml", {'search[postal_code]': postal_code})
        return [GoodreadsEvent(e) for e in resp['events']['event']]

    def recent_reviews(self):
        """Get the recent reviews from all members"""
        resp = self.request("/review/recent_reviews.xml", {})
        return [GoodreadsReview(r) for r in resp['reviews']['review']]

    def review(self, review_id):
        """Get a review"""
        resp = self.request("/review/show.xml", {'id': review_id})
        return GoodreadsReview(resp['review'])



"""Class for GoodreadsEvent"""

class GoodreadsEvent:
    def __init__(self, event_dict):
        self._event_dict = event_dict

    def __repr__(self):
        return self.title

    @property
    def gid(self):
        """ID for the event"""
        return self._event_dict['id']

    @property
    def title(self):
        """Title for the event"""
        return self._event_dict['title']

    @property
    def description(self):
        """Description for the event"""
        return self._event_dict['description']

    @property
    def link(self):
        """Link for the event"""
        return self._event_dict['link']

    @property
    def venue(self):
        """Venue for the event"""
        return self._event_dict['venue']

    @property
    def address(self):
        """Address for the event"""
        return self._event_dict['address']

    @property
    def city(self):
        """City for the event"""
        return self._event_dict['city']

    @property
    def postal_code(self):
        """Postal code for the event"""
        return self._event_dict['postal_code']

    @property
    def state_code(self):
        """State for the event"""
        return self._event_dict['state_code']

    @property
    def country_code(self):
        """Country code for the event"""
        return self._event_dict['country_code']

    @property
    def access(self):
        """Is event public or private?"""
        return self._event_dict['access']

    @property
    def event_type(self):
        """Type of the event"""
        return self._event_dict['event_type']

    @property
    def added_by(self):
        """User id for the event creator"""
        return self._event_dict['user_id']['#text']

    @property
    def image_url(self):
        """Image URL for the event"""
        return self._event_dict['image_url']

    @property
    def created_at(self):
        """Event created at"""
        return self._event_dict['created_at']['#text']

    @property
    def updated_at(self):
        """Event updated at"""
        return self._event_dict['updated_at']['#text']

    @property
    def reminder_at(self):
        """Reminder time for the event"""
        return self._event_dict['reminder_at']['#text']

    @property
    def rsvp_end_at(self):
        """RSVP for the event ends at"""
        return self._event_dict['rsvp_end_at']['#text']

    @property
    def start_at(self):
        """Event starts at"""
        return self._event_dict['start_at']['#text']

    @property
    def end_at(self):
        """Event ends at"""
        return self._event_dict['end_at']['#text']

    @property
    def attending_count(self):
        """Number of people attending"""
        return int(self._event_dict['event_attending_count']['#text'])

    @property
    def responses_count(self):
        """Number of poeple invited"""
        return int(self._event_dict['event_responses_count']['#text'])

    @property
    def resource(self):
        """Type and ID for the event resource"""
        return (self._event_dict['resouce_type'],
                self._event_dict['resouce_id']['#text'])



import user
import book
import comment

class GoodreadsUserStatus:
    def __init__(self, user_status_dict):
        self._user_status_dict = user_status_dict

    @property
    def gid(self):
        """Goodreads id of the status"""
        return self._user_status_dict['id']

    @property
    def work_gid(self):
        """Goodreads id of the work"""
        return self._user_status_dict['work_id']

    @property
    def header(self):
        """Header of the status"""
        return self._user_status_dict['header']

    @property
    def body(self):
        """Body of the status"""
        return self._user_status_dict['body']

    @property
    def page(self):
        """Page status"""
        return self._user_status_dict['page']

    @property
    def percent(self):
        """Percent completion of the book"""
        return self._user_status_dict['percent']

    @property
    def user(self):
        """User object for the status"""
        return user.GoodreadsUser(self._user_status_dict['user'])

    @property
    def book(self):
        """Book object for the status"""
        return book.GoodreadsBook(self._user_status_dict['book'])

    @property
    def comments(self):
        """Comments for the status"""
        return [comment.GoodreadsComment(comment_dict)
                for comment_dict in self._user_status_dict['comments']]

    @property
    def created_at(self):
        """User status created at"""
        return self._user_status_dict['created_at']

    @property
    def updated_at(self):
        """User status updated at"""
        return self._user_status_dict['updated_at']

    @property
    def likes_count(self):
        """How many likes this status has?"""
        return self._user_status_dict['likes_count']

    @property
    def comments_count(self):
        """How many comments this status has?"""
        return self._user_status_dict['comments_count']

    @property
    def liked(self):
        """Did you like this status"""
        return self._user_status_dict['liked']




from rauth.service import OAuth1Service, OAuth1Session
import xmltodict

class GoodreadsSession():
    """Handle OAuth sessions"""
    def __init__(self, client_key, client_secret, access_token=None,
                 access_token_secret=None):
        self.client_key = client_key
        self.client_secret = client_secret
        self.access_token = access_token
        self.access_token_secret = access_token_secret

    def oauth_init(self):
        """Start outh and return authorization url."""
        service = OAuth1Service(
            consumer_key=self.client_key,
            consumer_secret=self.client_secret,
            name='goodreads',
            request_token_url='http://www.goodreads.com/oauth/request_token',
            authorize_url='http://www.goodreads.com/oauth/authorize',
            access_token_url='http://www.goodreads.com/oauth/access_token',
            base_url='http://www.goodreads.com/'
        )
        request_token, request_token_secret = service.get_request_token(header_auth=True)
        auth_url = service.get_authorize_url(request_token)
        # Store service for finalizing
        self.request_token = request_token
        self.request_token_secret = request_token_secret
        self.service = service
        return auth_url

    def oauth_finalize(self):
        """Once the user authorizes access, save access tokens"""
        print self.request_token, self.request_token_secret
        self.session = self.service.get_auth_session(self.request_token,
                                                     self.request_token_secret)
        self.access_token = self.session.access_token
        self.access_token_secret = self.session.access_token_secret

    def oauth_resume(self):
        """Create session if access token and key are already available"""
        self.session = OAuth1Session(
            consumer_key=self.client_key,
            consumer_secret=self.client_secret,
            access_token=self.access_token,
            access_token_secret=self.access_token_secret)

    def get(self, path, params=None):
        """OAuth get request"""
        if not params:
            params = {}
        base = "http://www.goodreads.com/"
        resp = self.session.get(base + path, params=params)
        return xmltodict.parse(resp.content)['GoodreadsResponse']



"""Goodreads book class"""

import author

class GoodreadsBook:
    def __init__(self, book_dict, client):
        self._book_dict = book_dict
        self._client = client

    def __repr__(self):
        return self.title

    @property
    def gid(self):
        """Goodreads id of the book"""
        return self._book_dict['id']

    @property
    def title(self):
        """Title of the book"""
        return self._book_dict['title']

    @property
    def authors(self):
        """Authors of the book"""
        # Goodreads API returns a list if there are more than one authors,
        # otherwise, just the OrderedDict
        if type(self._book_dict['authors']['author']) == list:
            return [author.GoodreadsAuthor(author_dict, self._client)
                    for author_dict in self._book_dict['authors']['author']]
        else:
            return [author.GoodreadsAuthor(self._book_dict['authors']['author'],
                                           self._client)]

    @property
    def description(self):
        """Description of the book"""
        return self._book_dict['description']

    @property
    def average_rating(self):
        """Average rating of the book"""
        return self._book_dict['average_rating']

    @property
    def rating_dist(self):
        """Rating distribution of the book"""
        return self._book_dict['work']['rating_dist']

    @property
    def ratings_count(self):
        """Number of ratings for the book"""
        return self._book_dict['ratings_count']

    @property
    def text_reviews_count(self):
        """Number of text reviews for the book"""
        return self._book_dict['text_reviews_count']

    @property
    def num_pages(self):
        """Number of pages of the book"""
        return self._book_dict['num_pages']

    @property
    def popular_shelves(self):
        """Popular shelves for the book"""
        return self._book_dict['popular_shelves']

    @property
    def work(self):
        """Information on the original work"""
        return self._book_dict['work']

    @property
    def series_works(self):
        """Return series of the book"""
        return self._book_dict['series_works']

    @property
    def publication_date(self):
        """Publication month/day/year for the book"""
        return (self._book_dict['publication_month'],
                self._book_dict['publication_day'],
                self._book_dict['publication_year'])

    @property
    def publisher(self):
        """Publisher for the book"""
        return self._book_dict['publisher']

    @property
    def language_code(self):
        """Language code for the book"""
        return self._book_dict['language_code']

    @property
    def edition_information(self):
        """Edition information of the book"""
        return self._book_dict['edition_information']

    @property
    def image_url(self):
        """Image URL of the book"""
        return self._book_dict['image_url']

    @property
    def small_image_url(self):
        """Small image URL of the book"""
        return self._book_dict['small_image_url']

    @property
    def is_ebook(self):
        """Is this book an e-book?"""
        return self._book_dict['is_ebook']

    @property
    def format(self):
        """Book format"""
        return self._book_dict['format']

    @property
    def isbn(self):
        """ISBN of the book"""
        return self._book_dict['isbn']

    @property
    def isbn13(self):
        """ISBN13 of the book"""
        return self._book_dict['isbn13']

    @property
    def link(self):
        """Link for the book"""
        return self._book_dict['link']

    @property
    def reviews_widget(self):
        """Widget for reviews in HTML"""
        return self._book_dict['reviews_widget']

    @property
    def similar_books(self):
        """Return the list of similar books."""
        return self._book_dict['similar_books']['book']






"""Class for Goodreads comments"""

import user

class GoodreadsComment:
    """Goodreads comment class"""
    def __init__(self, comment_dict):
        self._comment_dict = comment_dict

    @property
    def gid(self):
        """Goodreads id of the comment"""
        return self._comment_dict['id']

    @property
    def body(self):
        """Body of the comment"""
        return self._comment_dict['body']

    @property
    def user(self):
        """User that made the comment"""
        return self._comment_dict['user']

    @property
    def created_at(self):
        """Comment created at"""
        return self._comment_dict['created_at']

    @property
    def updated_at(self):
        """Comment updated at"""
        return self._comment_dict['updated_at']



"""Class definition for owned books"""

import book

class GoodreadsOwnedBook:
    def __init__(self, owned_book_dict):
        self._owned_book_dict = owned_book_dict

    @property
    def gid(self):
        """Goodreads id of the owned book"""
        return self._owned_book_dict['id']['#text']

    @property
    def book(self):
        """Book owned"""
        return book.GoodreadsBook(self._owned_book_dict['book'])

    @property
    def review(self):
        """Review for the owned book"""
        return self._owned_book_dict['review']

    @property
    def current_owner(self):
        """Return current owner's id"""
        return self._owned_book_dict['current_owner_id']['#text']

    @property
    def original_purchase_date(self):
        """Date of purchase"""
        return self._owned_book_dict['original_purchase_date']

    @property
    def original_purchase_location(self):
        """Purchase location"""
        return self._owned_book_dict['original_purchase_location']

    def condition(self):
        """Condition of the book"""
        return self._owned_book_dict['condition']

    def link(self):
        """Linked for the owned book"""
        return self._owned_book_dict['link']



from setuptools import setup

setup(
    name='goodreads',
    description="Python wrapper for Goodreads API",
    long_description=open("README.rst").read(),
    author='Sefa Kilic',
    author_email='sefakilic@gmail.com',
    url='https://github.com/sefakilic/goodreads/',
    version='0.2.4',
    install_requires=['nose', 'xmltodict', 'requests', 'rauth'],
    packages=['goodreads'],
    scripts=[],
    license='MIT',
)



import book
import request
import group
import owned_book
import review

class GoodreadsUser():
    def __init__(self, user_dict, client):
        self._user_dict = user_dict
        self._client = client   # for later queries

    def __repr__(self):
        return self.user_name

    @property
    def gid(self):
        """Goodreads ID for the user"""
        return self._user_dict['id']

    @property
    def user_name(self):
        """Goodreads handle of the user"""
        return self._user_dict['user_name']

    @property
    def name(self):
        """Name of the user"""
        return self._user_dict['name']

    @property
    def link(self):
        """URL for user profile"""
        return self._user_dict['link']

    @property
    def image_url(self):
        """URL of user image"""
        return self._user_dict['image_url']

    @property
    def small_image_url(self):
        """URL of user image (small)"""
        return self._user_dict['small_image_url']

    def list_groups(self, page=1):
        """List groups for the user. If there are more than 30 groups, get them
        page by page."""
        resp = self._client.request("group/list/%s.xml" % self.gid, {'page':page})
        return resp['groups']['list']['group']

    def owned_books(self, page=1):
        """Return the list of books owned by the user"""
        resp = self._client.session.get("owned_books/user/%s.xml" % self.gid,
                                        {'page': page, 'format': 'xml'})
        return [owned_book.GoodreadsOwnedBook(d)
                for d in resp['owned_books']['owned_book']]

    def read_status(self):
        """Get the user's read status"""
        resp = self._client.request("read_statuses/%s" % self.gid, {})
        return resp['read_status']

    def reviews(self, page=1):
        """Get all books and reviews on user's shelves"""
        resp = self._client.session.get("/review/list.xml",
                                        {'v': 2, 'id': self.gid, 'page': page})
        return [review.GoodreadsReview(r) for r in resp['reviews']['review']]

    def shelves(self, page=1):
        """Get the user's shelves. This method gets shelves only for users with
        public profile"""
        resp = self._client.request("shelf/list.xml",
                                    {'user_id': self.gid, 'page': page})
        return resp['shelves']['user_shelf']

